/*
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 * 
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *          http://www.eclipse.org/legal/epl-v10.html
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.imp;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.AbstractAction;
import javax.swing.Icon;
import javax.swing.JFileChooser;
import javax.swing.filechooser.FileFilter;
import javax.swing.filechooser.FileNameExtensionFilter;
import javax.swing.filechooser.FileView;
import org.netbeans.api.project.Project;
import org.openide.awt.ActionID;
import org.openide.awt.ActionReference;
import org.openide.awt.ActionReferences;
import org.openide.awt.ActionRegistration;
import org.openide.filesystems.FileChooserBuilder;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileUtil;
import org.openide.util.Lookup;
import org.openide.util.NbBundle.Messages;

@ActionID(
    category = "File",
id = "nl.cloudfarming.client.imp.ImportAction")
@ActionRegistration(iconBase = "nl/cloudfarming/client/imp/import.png",
displayName = "#CTL_ImportAction")
@ActionReferences({
    @ActionReference(path = "Menu/File", position = 0)
})
@Messages({"CTL_ImportAction=Import", "ImportAction chooser title=Import file"})
public final class ImportAction extends AbstractAction {

    private final Project context;
    private final static Logger LOGGER = Logger.getLogger("nl.cloudfarming.client.imp.ImportAction");

    public ImportAction(Project context) {
        super(Bundle.ImportAction_chooser_title());
        this.context = context;
    }

    @Override
    public void actionPerformed(ActionEvent ev) {
        FileChooserBuilder builder = new FileChooserBuilder(ImportAction.class);
        builder.setTitle(Bundle.ImportAction_chooser_title());

        for (ImportProcessor importProcessor : Lookup.getDefault().lookupAll(ImportProcessor.class)) {
            FileFilter filter = new FileNameExtensionFilter(importProcessor.getFormatDescription(), importProcessor.getSupportedExtentions());
            builder.addFileFilter(filter);
        }
        final JFileChooser chooser = builder.createFileChooser();
        chooser.setMultiSelectionEnabled(true);
        chooser.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                if (JFileChooser.APPROVE_SELECTION.equals(e.getActionCommand())) {
                    for (File file : chooser.getSelectedFiles()) {
                        importFile(file);
                    }
                }
            }
        });
        chooser.setFileHidingEnabled(true);
        chooser.setFileView(new FileView() {
            @Override
            public Icon getIcon(File f) {
                Icon icon = super.getIcon(f);
                FileObject fileObject = FileUtil.toFileObject(f);
                if (fileObject != null) {
                    for (ImportProcessor importProcessor : Lookup.getDefault().lookupAll(ImportProcessor.class)) {
                        if (importProcessor.isSupported(fileObject)) {
                            icon = importProcessor.getIcon();
                            break;
                        }
                    }
                }
                return icon;
            }
        });
        chooser.showOpenDialog(null);


    }

    private void importFile(File file) {
        LOGGER.log(Level.FINEST, "starting import for selected file {0}", file.getAbsolutePath());
        // recurse if file is a folder
        if (file.isDirectory()) {
            for (File child : file.listFiles()) {
                importFile(child);
            }
        }
        // search for available processor
        for (ImportProcessor importProcessor : Lookup.getDefault().lookupAll(ImportProcessor.class)) {
            FileObject fo = FileUtil.toFileObject(file);
            if (fo != null && importProcessor.isSupported(fo)) {
                LOGGER.log(Level.FINEST, "Processor found for file {0}", importProcessor.toString());
                importProcessor.doImport(fo, context);
                break;
            }
        }
    }
}
